/**********************************************************************
  * File Name   : ircdchild.c
  * Function    : This module is a wrapper for when the server process
  *               forks. It will initate any dialogue between the srv
  *               and client.
  * HIstory     : Created 05 July 2002 jcorcora.
  *********************************************************************/


#include <stdio.h>
#include <crazyirc.h>
#include <string.h>
#include <errno.h>

/* Function Prototypes */
static int      check_dialogue      (char *,int);
static int      verify_command      (char *);
static void     act_on_command      (int,char *);
static void     get_user_details    (char *);
static void     get_nick_details    (char *);
static void     get_passwd          (char *);
static int      reply_msg_numeric   (int,char *);
static void     join_channel        (char *);
static void     reply_connect_reg   (void);
static void     reply_names_req     (void);
static void     reply_msg_string    (char *);

/* Constants. */
#define TOTALCMDS 5
#define TRY_REG 3

enum {
    NONE = 0,
    USER,
    NICK,
    PASSWD,
    JOIN,
    QUIT
};

/**********************************************************************
  * Function Name : start_dialogue
  * Arguments     : in  Void.
  *                 out Void.
  ********************************************************************/

void
start_dialogue (void)
{
    char *buf;
    char data_3[400];
    int rc;
    int loop = true;

    fprintf(stderr,"Client Connected.\n Starting Dialogue.\n");
    buf = data_3;
    while (loop == true)
    {
        rc = read(data.ssock,buf,400);
        strcat(buf+strlen(buf)+1,"\n"); 
        fprintf(stderr,"Read [%s][%d].\n",buf,rc);
        if (rc == false)
        {
            fprintf(stderr,"Disconnect.\n");
            loop = false;
        }
        else
        {
            rc = check_dialogue(buf,rc);
            if (rc == QUIT)
            {
                fprintf(stderr,"Quit Command Received.\n");
                loop = false;
            }
        }
    }
}


/********************************************************************
  * Function Name   : check_dialogue
  * Arguments       : int - Char * int
  *                   out - int
  *
  *
  *******************************************************************/
int
check_dialogue (char *string, int len)
{
    char *ptr;
    char *command;
    char temp;
    int  rc; 

    command = (char *) malloc(10); 
    ptr = (char *)malloc(len);
    
    strncpy(ptr,string,len);
    temp = 0x20; 
    
    command = strtok(ptr,&temp);
    fprintf(stderr,"Found command [%s]\n",command);
    
    if (command != NULL)
    {
        rc = verify_command(command);
    }
    else
    {
        rc = NONE;
    }
    if (rc == NONE || rc == QUIT )
    {
       fprintf(stderr,"Unknown Command.\n");
       goto exit_point;
    } 
    
    act_on_command(rc,string+(strlen(command)));

exit_point:
    return rc;
}


/*********************************************************************
  * Function Name   : verify_command.
  * Arguments       : int - char *
  *                   out - int
  *
  *
  ********************************************************************/

int 
verify_command (char *command)
{
    int rc = 0;
    int cmd = 0;
    int i;
    char command_allowed[TOTALCMDS+1][20]={"NULL","USER","NICK","PASS","JOIN","QUIT"};

    for (i = 0; i <= TOTALCMDS; i++)
    {
        rc = strcmp(command,command_allowed[i]);
        if (rc == 0)
        {
            fprintf(stderr,"Command [%s] = [%d].\n",command,i);
            cmd = i;
            i = TOTALCMDS + 1;
        }

    }
    return cmd;
} 

/**********************************************************************
  * Function Name : act_on_command
  * Arguments     : in - int char *
  *                 out - void.
  *
  *
  ********************************************************************/

void 
act_on_command (int command, char *string)
{
    fprintf(stderr,"Received string [%s].\n",string);
    while (*string == ' ')
    {
        string++;
    }
    switch(command)
    {
        case USER :
            get_user_details(string);
            break;
        case NICK :
            get_nick_details(string);
            break;
        case PASSWD :
            get_passwd(string);
            break;
        case JOIN : 
            join_channel(string);
            break;
        default : 
            fprintf(stderr,"In thoery Should Not be here.\n");
            break;
    }
}

/*********************************************************************
  * Function Name   : get_user_details
  * Arguments       : in - char *
  *                   out - void.
  ********************************************************************/
void
get_user_details(char *string)
{
    char *ptr;
    char cmmd[100];
    char *command = cmmd;
    int i; 
    int rc = 0;
    int temp;
    ptr = string;
    
    data.user_data.user_name = (char *)malloc(30);
    data.user_data.host_name = (char *)malloc(30);
    data.user_data.server_name = (char *)malloc(30);
    data.user_data.real_name = (char *)malloc(30);

    data.user_data.user_name = strtok(ptr," ");
    data.user_data.host_name = strtok(NULL," ");
    data.user_data.server_name = strtok(NULL," ");
    data.user_data.real_name = strtok(NULL,"\n");
    
    command = strtok(NULL,"\n");
    fprintf(stderr,"Command = [%s].\n",command); 
    fprintf(stderr,"username [%s]\n host_name [%s]\n server_name[%s]\n relaname [%s]\n",data.user_data.user_name,data.user_data.host_name,data.user_data.server_name ,data.user_data.real_name);

    if (command != NULL)
    { 
        check_dialogue(command,strlen(command));
    } 
    data.logged_in += 1;
    if (data.logged_in >= 2)
    {
        reply_connect_reg(); 
    }
    
}



/*********************************************************************
  * Function Name   : get_nick_details
  * Arguments       : in - char *
  *                   out - void.
  ********************************************************************/

void
get_nick_details (char *string)
{
    char *command;
    char cmd[100];

    command = cmd;
    
    command = strtok(string,"\n");
    data.user_data.nick_name = (char *) malloc (strlen(command));
    strncpy(data.user_data.nick_name,command,strlen(command) -1);
    fprintf(stderr,"Got a Nick name of [%s].\n",data.user_data.nick_name);
    command = strtok(NULL,"\n");
    if (command != NULL)
    {
       check_dialogue(command,strlen(command));
    } 
    data.logged_in++;
    if (data.logged_in >= 2)
    {
        reply_connect_reg();
    }
}


/*********************************************************************
  * Function Name   : get_passwd
  * Arguments       : in - char *
  *                   out - void.
  ********************************************************************/

void
get_passwd (char *string)
{
    char cmd[100];
    char *command = cmd;

    command = strtok(string,"\n");

    data.user_data.passwd = (char *) malloc (strlen(command));
    strcpy(data.user_data.passwd,command);
    fprintf(stderr,"Got a Password of [%s].\n",data.user_data.passwd);
    command = strtok(NULL,"\n");
    if (command != NULL)
    {
        check_dialogue(command,strlen(command));
    }
    data.logged_in++;
}

/*********************************************************************
  * Function Name   : reply_msg_numeric
  * Args            : in - int char *
  *                 : out - int
  ********************************************************************/

int 
reply_msg_numeric (int code,char *message)
{
    int is_msg = true;
    char buff[100];
    char *buf = buff;
    int rc = 0;
    
    fprintf(stderr,"Sending message to client.\n"); 
    fprintf(stderr,"nick= [%s].",data.user_data.nick_name); 
    if (message != NULL)
    {
        if (code < 100)
        {
        sprintf(buf,":bpc241 00%d %s %s\n",code,data.user_data.nick_name,message);
    
        }
        else
        {
            sprintf(buf,":bpc241 %d %s %s\n",code,data.user_data.nick_name,message);
        }
    }
    else
    {
        if (code < 100)
        {
            sprintf(buf,":bpc241 00%d %s\n",code,data.user_data.nick_name);
        }
        else
        {
            sprintf(buf,":bpc241 %d %s\n",code,data.user_data.nick_name);
    
        }
    }
    rc = write_string(buf);
    return rc;
}
    
    

/**********************************************************************
  * Function Name   : join_channel
  * Arguments       : in - char *
  *                   out - void
  *
  *********************************************************************/

void
join_channel (char *message)
{
    char reply[200];
    char *command;
    char cmd[100];
    char channel[100];
    char *ptr = reply;
    char buf[200];
    fprintf(stderr,"Message = [%s].\n",message);
    command = cmd;
    command = strtok(message,"\n");
    
    while(*command == ' ')
    {
        command++;
    }
    strncpy(channel,command,strlen(command));
     
    ptr = strip_term(channel);
    
    fprintf(stderr,"Going with channel = [%s].\n",ptr);
    strcpy(data.channel, ptr);
    sprintf(buf,":%s JOIN %s",data.user_data.nick_name,data.channel); 
    reply_msg_string(buf);
    sprintf(reply,"%s :This is a test channel.",data.channel);
    reply_msg_numeric(RPL_TOPIC,reply);

    reply_names_req();
    command = strtok(NULL,"\n");

    if (command != NULL)
    {
        check_dialogue(command,strlen(command));
    }
}

/*********************************************************************
  * Function Name   : reply_msg_string
  * Arguments       : in - char *
  *                   out - void
  ********************************************************************/
void
reply_msg_string (char *message)
{
   int rc = 0;
   char *buf;

   buf = (char *)malloc(strlen(message)+1);
   sprintf(buf,"%s\n",message); 
   rc = write_string(buf); 
   
}

/*********************************************************************
  * Function Name   : reply_names_req
  * Arguments       : in = out = void
  *
  ********************************************************************/
void
reply_names_req (void)
{
    char reply[200];


    sprintf(reply,"* %s : @%s +corcoraj",data.channel,data.user_data.nick_name);
    reply_msg_numeric(RPL_NAMREPLY,reply);
    sprintf(reply,"%s :End of /Names list",data.channel);
    reply_msg_numeric(RPL_ENDOFNAMES,reply);
    
}
        


/*********************************************************************
  * Function Name   : reply_connect_reg
  * Arguments       : in = out = void
  *
  ********************************************************************/
void
reply_connect_reg (void)
{
    char cmd[200];
    char *command = cmd;
    
    fprintf(stderr,"Sending RPL_WELCOME.\nRPL_YOURHOST\nRPL_CREATED\nRPL_MYINFO.\n"); 
    reply_msg_numeric(RPL_WELCOME,":Welcome to corcoraj test server");
    reply_msg_numeric(RPL_YOURHOST,":Your host is bpc241 running 1.2");
    reply_msg_numeric(RPL_CREATED,":Server was created 123457");
    reply_msg_numeric(RPL_MYINFO,"bpc241 1.2 aoOirw abeiIklmnoOpqrstv");
    reply_msg_numeric(RPL_MOTDSTART,":- bpc241 Message of the Day.-");
    reply_msg_numeric(RPL_MOTD,":- This is a test MOTD");
    reply_msg_numeric(RPL_ENDOFMOTD,":End MOTD command");    
}
