/*******************************************************************
  * File Name   : ircdpidfile.c
  * Function    : This module will check to see if a instance is running
                  If not it will create a pid lock file.
  * History     : created 4th of July 2002. jcorcora
  ******************************************************************/
#include "crazyirc.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>



const char pid_file[] = "/tmp/crazyirc.pid";

/******************************************************************
  * Function Name       : Check_pid_file.
  * Arguments Taken     : Void;
  * Arguments Returned  : 0 if process not already running,
  *                       1 if there is an instance of ircd running
  *****************************************************************/
int
check_pid_file (void) {
    
    FILE *fp = NULL;
    int rc = 0;
    int pid = 0;
    struct stat buf;
    char file_data[20];

    
    rc = stat((const char *)pid_file,&buf);
    if (rc != false)
    {
        fprintf(stderr,"Pid File Dosen't Exsist.\n");
        rc = false;
        goto exit_pid_check;
    }
    else
    {
        fp = fopen(pid_file,"r");
        if (fp == NULL)
        {
            fprintf(stderr,"Error opening file....\n");
            errwin(errno);
            rc = false;
            goto exit_pid_check;
        }
        fgets(file_data,20,fp);
        pid = atoi(file_data);
        if ( kill((pid_t)pid,0) == -1) {
            if (errno == ESRCH)
            {
                fprintf(stderr,"Process [%d] not running.\n",pid);
                rc = false;
                goto exit_pid_check;
            }
            fprintf(stderr,"An instance of Crazy IRCD is running.\n");
            rc = true;
            goto exit_pid_check;
        }    
    }
exit_pid_check:
    if (fp != NULL)
    {
        fclose(fp);
    }
    return (rc);
}/********************************************************************
  * Function Name : create_pid_file 
  * Arguments     : in = out = void
  ********************************************************************/

void
create_pid_file (void)
{
    FILE *fp;
    pid_t mypid;
    char data[20];
    int rc=0;
    
    fp = fopen (pid_file,"w");
    
    if (fp == NULL)
    {
        errwin(errno);
        exit(-1);
    }
    
    mypid = getpid();
    rc = sprintf(data,"%d",(int)mypid);
    fwrite(data,strlen(data),1,fp);
    
    if (rc < false )
    {
        fprintf(stderr,"Error writing pid.\n");
    }
    else
    {
        fprintf(stderr,"Pid file written.\n");
    }
    fclose(fp);
}




