/********************************************************************
  * File Name   : ircdsocket.c
  * Function    : This module acts as a wrapper for all the TCP actions.
  * History     : created 4th of July 2002 jcorcora.
  *******************************************************************/

#include "crazyirc.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <sys/wait.h>
#include <netdb.h>
#include <errno.h>
#include <unistd.h>

#define MAX_Q_LEN 1000
const char  TRANSPORT[] =  "tcp";

static int  passiveSock             (int);




/********************************************************************
  * Function Name : connect_and_wait
  * Arguments     : in - int port to listen on.
  *                 out - void.
*********************************************************************/
void
connect_and_wait (int port)
{
    int msock;
    int alen;
    struct sockaddr_in fsin;
    int rc = 0;
    
    msock = passiveSock(port);
    
    while (rc == false)
    {
        alen = sizeof(fsin);
        fprintf(stderr,"Listening for Connections ....\n");
        data.ssock = accept(msock,(struct sockaddr *)&fsin, &alen);
        if (data.ssock < false)
        {
            errwin(errno);
        }

        switch (fork())
        {
            case 0  : /* Child process */
                (void) close(msock);
                start_dialogue();
                (void) close(data.ssock);
                rc = true;
                break;
            default : /* Parent Process */
                fprintf(stderr,"In the Parent.\n");
                (void) close(data.ssock);
                rc = false;
                break;
            case -1 :
                errwin(errno);
        }
    }
}
                

/**********************************************************************
  * Function Name   : passiveSock
  * Arguments       : in - int - port number to connect to
  *                 : Out- int - socket that we can accept con on.
  *********************************************************************/
int
passiveSock (int port)
{

    struct protoent *ppe;
    struct sockaddr_in sin;
    int     s;

    memset(&sin,0,sizeof(sin));

    sin.sin_family = PF_INET;
    sin.sin_addr.s_addr = INADDR_ANY;

    if ((sin.sin_port = htons((u_short)port)) == false)
    {
        fprintf(stderr,"Can't get service entry.\n");
        exit(-1);
    }
    
    if ((ppe = getprotobyname(TRANSPORT))== false)
    {
        fprintf(stderr,"Can't get protocol TCP.\n");
        errwin(errno);
        exit(-1);
    }

    s = socket(PF_INET, SOCK_STREAM, ppe->p_proto);
    if (s < false)
    {
        fprintf(stderr,"Error allocating socket.\n");
        errwin(errno);
        exit(-1);
    }
    if (bind(s, (struct sockaddr *) &sin,sizeof(sin)) < false)
    {
        fprintf(stderr,"Error Binding to socket.\n");
        errwin(errno);
        exit(-1);
    }
    listen(s,MAX_Q_LEN); 
    return s;          
}


int
write_string (char *buf)
{
    int rc;
    
    fprintf(stderr,"\nWritting the following to the ssock [%s].\n",buf);
    
    rc = write(data.ssock,buf,strlen(buf)); 
    
    fprintf(stderr,"Wrote [%d] bytes.\n",rc); 
    
    if (rc == -1)
    {
        errwin(errno);
        rc = 0;
    }
    return rc;
}
       

